﻿using System;
using System.Collections.Generic;
using System.Collections.ObjectModel;
using System.Diagnostics;
using System.Drawing;
using System.Drawing.Imaging;
using System.IO;
using System.Web.UI;
using PdfSharp;
using PdfSharp.Pdf;
using ThinkGeo.MapSuite.Core;
using ThinkGeo.MapSuite.WebEdition;

namespace DisplayASimpleMap
{
    public partial class _Default : System.Web.UI.Page
    {
        protected void Page_Load(object sender, EventArgs e)
        {
            if (!Page.IsPostBack)
            {
                Map1.MapBackground.BackgroundBrush = new GeoSolidBrush(GeoColor.FromHtml("#E5E3DF"));
                Map1.MapUnit = GeographyUnit.DecimalDegree;

                ShapeFileFeatureLayer worldLayer = new ShapeFileFeatureLayer(MapPath("~/Data/USStates.shp"));
                worldLayer.ZoomLevelSet.ZoomLevel01.ApplyUntilZoomLevel = ApplyUntilZoomLevel.Level20;
                worldLayer.ZoomLevelSet.ZoomLevel01.DefaultAreaStyle = AreaStyles.Country1;
                Map1.StaticOverlay.IsBaseOverlay = false;
                Map1.StaticOverlay.Layers.Add("ShapeLayer", worldLayer);

                Map1.CurrentExtent = new RectangleShape(-172.61778930575, 88.071925200522, -63.01817993075, 4.839503325522);

                ScaleBarAdornmentLayer scaleBar = new ScaleBarAdornmentLayer();
                scaleBar.Location = AdornmentLocation.UpperRight;
                Map1.AdornmentOverlay.Layers.Add("ScaleBar", scaleBar);

                SimpleMarkerOverlay markerOverlay = new SimpleMarkerOverlay();
                Map1.CustomOverlays.Add(markerOverlay);
            }
        }

        protected void Button_Click(object sender, EventArgs e)
        {
            using (PdfDocument pdfDocument = new PdfDocument())
            {
                PdfPage pdfPage = pdfDocument.AddPage();
                pdfPage.Orientation = PageOrientation.Landscape;
                pdfPage.Size = PageSize.A0;
                PdfGeoCanvas pdfGeoCanvas;
                Stream stream = GetImageStream();
                pdfGeoCanvas = new PdfGeoCanvas();
                pdfGeoCanvas.BeginDrawing(pdfPage, Map1.CurrentExtent, Map1.MapUnit);
                pdfGeoCanvas.DrawScreenImage(new GeoImage(stream), (float)Map1.WidthInPixels / 2, (float)Map1.HeightInPixels / 2, (float)Map1.WidthInPixels, (float)Map1.HeightInPixels, DrawingLevel.LevelOne, 0, 0, 0);
                pdfGeoCanvas.EndDrawing();
                string fileName = "printing.pdf";
                pdfDocument.Save(fileName);
                Process.Start(fileName);
            }
        }

        private Stream GetImageStream()
        {
            using (Bitmap bitmap = new Bitmap((int)Map1.WidthInPixels, (int)Map1.HeightInPixels))
            {
                MemoryStream stream = new MemoryStream();
                GdiPlusGeoCanvas geoCanvas = new GdiPlusGeoCanvas();

                geoCanvas.BeginDrawing(bitmap, Map1.CurrentExtent, GeographyUnit.Meter);
                DrawShapeFileFeatureLayer(geoCanvas);
                DrawScaleBar(geoCanvas);
                DrawTitle(geoCanvas);
                DrawMarkers(bitmap);
                geoCanvas.EndDrawing();

                stream = new MemoryStream();
                bitmap.Save(stream, ImageFormat.Png);
                return stream;
            }
        }

        private void DrawScaleBar(GdiPlusGeoCanvas geoCanvas)
        {
            Layer scaleBarLayer = Map1.AdornmentOverlay.Layers["ScaleBar"];
            scaleBarLayer.Open();
            scaleBarLayer.Draw(geoCanvas, new Collection<SimpleCandidate>());
        }

        private void DrawShapeFileFeatureLayer(GdiPlusGeoCanvas geoCanvas)
        {
            Layer shapeLayer = Map1.StaticOverlay.Layers["ShapeLayer"];
            shapeLayer.Open();
            shapeLayer.Draw(geoCanvas, new Collection<SimpleCandidate>());
        }

        private void DrawMarkers(Bitmap bitmap)
        {
            Graphics g = Graphics.FromImage(bitmap);
            foreach (var item in ((SimpleMarkerOverlay)Map1.CustomOverlays[0]).Markers)
            {
                Bitmap markerBitmap = new Bitmap(Server.MapPath(item.WebImage.ImageVirtualPath));
                double x = (item.Position.X - Map1.CurrentExtent.UpperLeftPoint.X) / Map1.CurrentExtent.Width * Map1.WidthInPixels;
                double y = (item.Position.Y - Map1.CurrentExtent.LowerRightPoint.Y) / Map1.CurrentExtent.Height * Map1.HeightInPixels;
                g.DrawImage(markerBitmap, (int)x, (int)(Map1.HeightInPixels - y));
            }
        }

        private void DrawTitle(GdiPlusGeoCanvas geoCanvas)
        {
            InMemoryFeatureLayer inm = (InMemoryFeatureLayer)Map1.StaticOverlay.Layers["InMemoryLayer"];
            inm.Open();
            inm.Draw(geoCanvas, new Collection<SimpleCandidate>());
        }

        protected void Map1_Click(object sender, MapClickedEventArgs e)
        {
            Marker marker = new Marker(e.Position);
            marker.WebImage = new WebImage("/theme/default/img/marker_blue.gif");
            ((SimpleMarkerOverlay)Map1.CustomOverlays[0]).Markers.Add(marker);
        }

        protected void titleDisplayButton_Click(object sender, EventArgs e)
        {
            RectangleShape pageBoundingbox = Map1.CurrentExtent;
            PointShape labelCenter = new PointShape();
            double realX = pageBoundingbox.UpperLeftPoint.X;

            GdiPlusGeoCanvas geocanvas = new GdiPlusGeoCanvas();
            float centerX = geocanvas.MeasureText(titleTextBox.Text, new GeoFont("Arial", 20, DrawingFontStyles.Bold)).Width / 2;
            double ratio = centerX / Map1.WidthInPixels * 2;
            double width = pageBoundingbox.Width / 2;
            realX = width - pageBoundingbox.Width * ratio / 2 + pageBoundingbox.UpperLeftPoint.X;

            labelCenter.X = realX;
            labelCenter.Y = (pageBoundingbox.UpperRightPoint.Y + pageBoundingbox.LowerRightPoint.Y) / 2;
            Dictionary<string, string> columnValues = new Dictionary<string, string>();
            columnValues.Add("Label", titleTextBox.Text);
            Feature feature = new Feature(labelCenter, columnValues);

            InMemoryFeatureLayer inm = new InMemoryFeatureLayer();
            inm.Open();
            inm.Columns.Add(new FeatureSourceColumn("Label"));
            inm.DrawingMarginPercentage = 1000;
            inm.FeatureSource.BeginTransaction();
            inm.FeatureSource.AddFeature(feature);
            inm.FeatureSource.CommitTransaction();
            inm.ZoomLevelSet.ZoomLevel01.ApplyUntilZoomLevel = ApplyUntilZoomLevel.Level20;
            inm.ZoomLevelSet.ZoomLevel01.DefaultTextStyle = TextStyles.CreateSimpleTextStyle("Label", "Arial", 20, DrawingFontStyles.Bold, GeoColor.SimpleColors.Black);
            if (Map1.StaticOverlay.Layers.Contains("InMemoryLayer"))
            {
                Map1.StaticOverlay.Layers.Remove("InMemoryLayer");
            }
            Map1.StaticOverlay.Layers.Add("InMemoryLayer", inm);
        }
    }
}
